#include "ut_telno_epoc.hpp"

#include <cntfield.h>
#include <cntfldst.h> 
#include <cntitem.h>

// The caller is to free any returned object.
static CContactItem* FindContactItemByPhoneNoL(CContactDatabase& database,
					       const TDesC& phoneNo)
{
  CContactItem* item = NULL;

  if (phoneNo.Length() > 0)
  {
    // This ought to (typically) be enough for a single match. If not, too bad.
    const TInt KNumberOfDigitsToMatch = 8;

    CContactIdArray* idArray = database.MatchPhoneNumberL(phoneNo, 
							  KNumberOfDigitsToMatch);
    CleanupStack::PushL(idArray);
    if (idArray->Count() == 1) {
      // We only take one match, whatever it happens to be.
      TContactItemId itemId = (*idArray)[0];
      TRAPD(errCode, item = database.ReadMinimalContactL(itemId));
      if (errCode) {
	if (errCode != KErrNotFound) 
	  User::Leave(errCode);
      }
    }
    CleanupStack::PopAndDestroy(idArray);
  }

  return item; // may be NULL
}

static HBufC* GetNameFromContactItemL(TDes& aBuffer,
				      const CContactItem& item)
{
  HBufC* newBuffer = NULL;

  CContactItemFieldSet& fieldSet = item.CardFields();

  TInt givenIx = fieldSet.Find(KUidContactFieldGivenName);
  TBool gotGiven = (givenIx != KErrNotFound);

  TInt familyIx = fieldSet.Find(KUidContactFieldFamilyName);
  TBool gotFamily = (familyIx != KErrNotFound);

  _LIT(KSpace, " ");

  TInt len = 0;

  if (gotGiven) {
    len += fieldSet[givenIx].TextStorage()->Text().Length();
  }
  if (gotGiven && gotFamily) {
    len += 1;
  }
  if (gotFamily) {
    len += fieldSet[familyIx].TextStorage()->Text().Length();
  }

  TDes* sp = NULL;
  TPtr newPtr(NULL, 0);
  if (len > aBuffer.MaxLength()) {
    newBuffer = HBufC::NewL(len + 1);
    CleanupStack::PushL(newBuffer);
    newPtr.Set(newBuffer->Des());
    sp = &newPtr;
  } else {
    aBuffer.Zero();
    sp = &aBuffer;
  }

  if (gotGiven) {
    sp->Append(fieldSet[givenIx].TextStorage()->Text());
  }
  if (gotGiven && gotFamily) {
    sp->Append(KSpace);
  }
  if (gotFamily) {
    sp->Append(fieldSet[familyIx].TextStorage()->Text());
  }

  if (newBuffer)
    CleanupStack::Pop(newBuffer);

  return newBuffer;
}

// The caller is to free any returned buffer. A new buffer is only
// returned if the passed buffer is not big enough.
HBufC* GetContactNameByPhoneNoL(CContactDatabase& database,
				TBool& aFound,
				TDes& aBuffer,
				const TDesC& phoneNo)
{
  CContactItem* item = FindContactItemByPhoneNoL(database, phoneNo);
  if (!item) {
    aFound = EFalse;
    return NULL;
  }
  CleanupStack::PushL(item);

  HBufC* newBuffer = GetNameFromContactItemL(aBuffer, *item);

  CleanupStack::PopAndDestroy(item);

  aFound = ETrue;
  return newBuffer;
}

/**

ut_telno_epoc.cpp

Copyright 2009 Helsinki Institute for Information Technology (HIIT)
and the authors. All rights reserved.

Authors: Tero Hasu <tero.hasu@hut.fi>

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

 **/
