#lang racket

#|

This file defines the API that all the variant specifications of this
project must implement.

|#

(require "konffaa/util.rkt")
(require "konffaa/variant.rkt")

;;; 
;;; base variants
;;; 

(define-variant* project-variant% variant%
  (super-new)

  ;; general...
  
  (define/public (platform) 'symbian)

  (define/public (platform-str.attr) (symbol->string (platform)))

  (define/public (app-basename.attr) "smsexporter")
  
  (define/public (app-name.attr) "SMS Exporter")

  (define/public (vendor-name.attr) "Tero Hasu")

  (define/public (major-version.attr) 1)
  
  (define/public (minor-version.attr) 10)
  
  (define/public (version100.attr)
    (+ (* (major-version.attr) 100) (minor-version.attr)))
  
  (define/public (version-string.attr)
    (real->decimal-string
     (+ (major-version.attr) (/ (minor-version.attr) 100)) 2))

  ;; platforms...
  
  (define/public (is-symbian.attr) (eq? (platform) 'symbian))

  ;; features...
  
  (define/public (feature-debugging.attr)
    #f)

  ) ;; end project-variant%

(define-variant* symbian-variant% project-variant%
  (super-new)
  
  ;; platform...
  
  (define/public (s60-vernum.attr) 30)

  ;; ELF file format.
  ;; E32 image format.
  ;; ARM EABI binary interface.
  (define/public (is-elf.attr)
    (>= (s60-vernum.attr) 30))

  ;; PE file format.
  ;; E32 image format.
  (define/public (is-pe.attr)
    (not (is-elf.attr)))
  
  (define/public (have-flogger.attr)
    (>= (s60-vernum.attr) 20))
  
  ;; SDK...
  
  (define/public (kit-vernum.attr) (s60-vernum.attr))

  (define/public (kit-name.attr)
    (format "s60_~a" (kit-vernum.attr)))

  ;; app info...
  
  (define/public (app-short-name.attr) "SMSExport")

  (define/public (app-long-name.attr) (send this app-name.attr))

  (define/public (uid-v8) #x10206ba3) ;; allocated from Symbian
  
  (define/public (uid-v9) #xf0206ba3) ;; derived from the above

  (define/public (uid) (if (is-pe.attr) (uid-v8) (uid-v9)))
  
  (define/public (uid-num.attr)
    (hexnum (uid)))

  (define/public (uid-str.attr)
    (format "~x" (uid)))

  ;; Need something fairly unique. Typically includes UID, but would
  ;; make UID harder to change, and we have no single UID.
  (define/public (unique-basename.attr)
    (format "~a_~a"
            "maebian"
            (send this app-basename.attr)
            ;;(uid-str.attr)
            ))

  ;; certification...

  (define/public (signed.attr)
    (if (< (s60-vernum.attr) 30) #f #t))
  
  (define/public (cert-name.attr)
    (send this app-basename.attr))
  
  (define/public (capabilities.attr)
    '(ReadUserData WriteUserData))
  
  ;; tools and libraries...
  
  (define/public (with-gnupoc.attr) #t)

  (define/public (with-sbsv1.attr) #t)

  ;; features...

  (define/public (abld-architecture.attr)
    (if (is-pe.attr) "armi" "gcce"))
  
  (define/public (abld-variant.attr)
    (if (send this feature-debugging.attr) "udeb" "urel"))

  (define/public (feature-logging.attr)
    (and (send this feature-debugging.attr)
         (have-flogger.attr)))
  
  ) ;; end symbian-variant%

#|

Copyright 2009 Helsinki Institute for Information Technology (HIIT)
and the authors. All rights reserved.

Authors: Tero Hasu <tero.hasu@hut.fi>

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

|#
